﻿using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace RH.NoBlogCommentSpam
{
    public class JobActivation : SPFeatureReceiver
    {
        private const string _JobTitle = "NoBlogCommentSpam";

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
            // remove job
            DeleteJob(properties);
        }

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // install Timer Job
            CreateJob(properties);
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            // remove WebApplication Settings if feature is deactivated
            var webApplication = properties.Feature.Parent as SPWebApplication;
            if (webApplication == null) return;

            //remove job if there is no other webapplication with this feature enabled
            bool stillUsed = false;
            var service = webApplication.Farm.Services.GetValue<SPWebService>("");
            foreach (SPWebApplication application in service.WebApplications)
            {
                if (application.Features[new Guid("EDC7FD11-75E1-4d81-88FD-A48101F14758")] != null)
                {
                    stillUsed = true;
                    break;
                }
            }

            if (stillUsed)
                DeleteJob(properties);
        }

        #region Job
        private static void CreateJob(SPFeatureReceiverProperties properties)
        {
            try
            {
                var webApplication = properties.Feature.Parent as SPWebApplication;
                if (webApplication == null) return;

                SPJobDefinitionCollection jobDefinitions = webApplication.JobDefinitions;

                if (Common.ContainsJob(jobDefinitions, _JobTitle)) return;

                SPJobDefinition job = new NoBlogCommentSpam(_JobTitle, webApplication) { Title = _JobTitle };
                var myMinuteSchedule = new SPMinuteSchedule {BeginSecond = 0, EndSecond = 59, Interval = 5};
                job.Schedule = myMinuteSchedule;

                Common.WriteULSVerbose(string.Format("Creating job definition for '{0}'", webApplication.Name));
                webApplication.JobDefinitions.Add(job);
                webApplication.Update();
            }
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }

        private static void DeleteJob(SPFeatureReceiverProperties properties)
        {
            try
            {
                var webApplication = properties.Feature.Parent as SPWebApplication;
                if (webApplication == null) return;

                foreach (var jobDefinition in webApplication.JobDefinitions)
                {
                    if (jobDefinition.Title != _JobTitle) continue;

                    Common.WriteULSVerbose(string.Format("Removing job definition from '{0}'", webApplication.Name));
                    jobDefinition.Delete();
                    webApplication.Update();
                }
            }
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }
        #endregion
    }
}